import styles from "../index.module.css";
import React, { useState} from 'react';
import { Outlet, Link} from "react-router-dom";
import mqtt from 'precompiled-mqtt';
import {creerSensor, existSensor, updateSensor, Sensors} from '../sensor.js';

export default function Root({sensors, setSensors}) {
  const [brokerUrl, setBrokerUrl] = useState('');

  const topic = 'value/#';
  const [client, setClient] = useState(null);

  const [isConnected, setIsConnected] = useState(false);

  const handleUrlChange = (e) => {
    const url = e.target.value;
    setBrokerUrl(url);
  };

  const handleDisconnect = () => {
    if(isConnected) {
      client.end();
      setIsConnected(false);
    }
  };

  const handleConnect = () => {
    if(!isConnected) {
      const newClient = mqtt.connect(brokerUrl);
      newClient.on('connect', () => {
        console.log(`Connected to MQTT broker ${brokerUrl}`);
        newClient.subscribe(topic, (err) => {
          if (err) {
            console.error('Error subscisConnectedribing to topic', err);
          } else {
            console.log(`Subscribed to topic ${topic}`);
          }
        });
      });

      newClient.on('message', (topic, message) => {
        const sensorData = JSON.parse(message.toString());
        const name = sensorData.name;
        const data = sensorData.value;
        if(existSensor(name)){
          updateSensor(name, data);
        }else {
          creerSensor(sensorData);
        }

        setSensors((prevSensors) => {
          const existingSensor = prevSensors.find((sensor) => sensor.name === name);
          if (existingSensor) {
            const updatedSensors = [...prevSensors];
            updatedSensors.find((sensor) => sensor.name === name).data.push(data);
            updatedSensors.find((sensor) => sensor.name === name).value = data;
            return updatedSensors;
          } else {
            sensorData.data =[];
            sensorData.data.push(data);
            return [...prevSensors, sensorData];
          }
        });
      });
      setClient(newClient);
    }
    setIsConnected(true);
  };
  function navLinkClicked(event) {
    event.preventDefault();
    const clickedLink = event.target;
    clickedLink.style.color = '#87CEEB';
    if (window.lastClickedNavbarLink && window.lastClickedNavbarLink !== clickedLink) {
      window.lastClickedNavbarLink.style.color = '';
    }
    window.lastClickedNavbarLink = clickedLink;
  }
  
  function sensorLinkClicked(event) {
    const clickedSensorLink = event.target;
    clickedSensorLink.style.color = '#87CEEB';
    if (window.lastClickedSensorLink && window.lastClickedSensorLink !== clickedSensorLink) {
      window.lastClickedSensorLink.style.color = '';
    }
    window.lastClickedSensorLink = clickedSensorLink;
    // On recupère la dernière nav bar link et on mets à jour sa couleur
    if (window.lastClickedNavbarLink) {
      window.lastClickedNavbarLink.style.color = '#87CEEB';
    }
  }
  
  return (
    <>
      <div className={styles.wrapper}>
        <div className={styles.header}>
          <div className={styles.navbar}>
            <Link onClick={navLinkClicked} to={`/`}>Real time sensors</Link>
            <a href={window.location.pathname} onClick={navLinkClicked}>Other</a>
            <a href={window.location.pathname} onClick={navLinkClicked}>Foo</a>
          </div>
        </div>
        <div className={styles.sidebar}>
          <h1 className={styles.sensors}>Sensors</h1>
          <div className={styles.listSensors}>
            <nav>
              {sensors.length ? (
                <ul>
                  {sensors.map((sensor) => (
                    <li key={sensor.id}>
                      <Link onClick={sensorLinkClicked} to={`/${sensor.name}`}>
                        {sensor.name ? (
                      <>
                        {sensor.name}
                      </>
                    ) : (
                      <i>No Name</i>
                    )}{" "}
                  </Link>
                </li>
              ))}
            </ul>
          ) : (
            <p>
              <i>No sensors</i>
            </p>
          )}
            </nav>
          </div>
        </div>
        <div className={styles.divUrl}>
          <h4 className={styles.mqtt}>MQTT Brocker:</h4>
          <input type="text" className={styles.urlInput} onChange={handleUrlChange} value={brokerUrl} />
          {isConnected ? (
              <button onClick={handleDisconnect}>Disconnect</button>
          ) : (
              <button onClick={handleConnect}>Connect</button>
          )}
        </div>
        <div className={styles.details}>
          <Outlet />
        </div>
      </div>
    </>
  );
}
